const assert = require('assert')
const isDev = require('electron-is-dev')
const ms = require('ms')
const path = require('path')
const fs = require('fs')

const supportedPlatforms = ['darwin', 'win32']

module.exports = function updater (opts = {}) {
  // check for bad input early, so it will be logged during development
  opts = validateInput(opts)

  // don't attempt to update during development
  if (isDev) {
    const message = 'update-electron-app config looks good; aborting updates since app is in development mode'
    opts.logger ? opts.logger.log(message) : console.log(message)
    return
  }

  opts.electron.app.isReady()
    ? initUpdater(opts)
    : opts.electron.app.on('ready', () => initUpdater(opts))
}

function initUpdater (opts) {
  const { updateInterval, logger, electron, autoUpdater } = opts
  const { dialog } = electron

  function log (...args) {
    logger.info(...args)
  }

  // exit early on unsupported platforms, e.g. `linux`
  if (typeof process !== 'undefined' && process.platform && !supportedPlatforms.includes(process.platform)) {
    log(`Electron's autoUpdater does not support the '${process.platform}' platform`)
    return
  }

  autoUpdater.on('update-downloaded', (event, releaseNotes, releaseName, releaseDate, updateURL) => {
    log('update-downloaded', arguments)

    const dialogOpts = {
      type: 'info',
      buttons: ['Restart', 'Later'],
      title: 'Application Update',
      message: process.platform === 'win32' ? releaseNotes : releaseName,
      detail: 'A new version has been downloaded. Restart the application to apply the updates.'
    }

    dialog.showMessageBox(dialogOpts, (response) => {
      if (response === 0) autoUpdater.quitAndInstall()
    })
  })

  // check for updates right away and keep checking later
  autoUpdater.checkForUpdates()
  setInterval(() => { autoUpdater.checkForUpdates() }, ms(updateInterval))
}

function validateInput (opts) {
  const defaults = {
    updateInterval: '10 minutes',
    logger: console
  }
  const { updateInterval, logger, autoUpdater } = Object.assign({}, defaults, opts)

  // allows electron to be mocked in tests
  const electron = opts.electron || require('electron')

  assert(
    autoUpdater,
    'autoUpdater is required'
  )

  assert(
    typeof updateInterval === 'string' && updateInterval.match(/^\d+/),
    'updateInterval must be a human-friendly string interval like `20 minutes`'
  )

  assert(
    ms(updateInterval) >= 5 * 60 * 1000,
    'updateInterval must be `5 minutes` or more'
  )

  assert(
    logger && typeof logger.log,
    'function'
  )

  return { autoUpdater, updateInterval, logger, electron }
}
